@@ -1,6 +1,6 @@
 use std::time::{Duration, Instant};
 
-use crate::{executor::execute, io_handler::IoHandler};
+use crate::{execute_many::execute_many, io_handler::IoHandler};
 use brainease_lexer::syntax::{CellPosition, CellValue, Instruction};
 
 /// A struct representing the result after parsing a whole instruction.
@@ -31,22 +31,10 @@ impl<I: IoHandler> Runtime<I> {
   pub fn run(&mut self) -> Result<Duration, I::Err> {
     let now = Instant::now();
 
-    let mut current_instruction = 0;
-    let runtime = &mut self.clone();
-
-    while current_instruction < self.instructions.len() {
-      execute(&self.instructions[current_instruction], runtime)?;
-      current_instruction += 1;
-    }
+    execute_many(&self.instructions.clone(), self)?;
 
     self.io_handler.flush()?;
 
-    // FIXME: Preserve state after execution.
-    // Without having to clone everything :P
-    self.io_handler = runtime.io_handler.clone();
-    self.memory = runtime.memory.clone();
-    self.instructions = runtime.instructions.clone();
-
     Ok(now.elapsed())
   }
 }
